% Author: Kaiyao WANG
% Date: 2023.03
% This data is referenced in our companion paper "Adaptive Weighted K-Nearest Neighbor Trilateration Algorithm for Visible Light Positioning"

clear all
close all
clc
tic

% Simulation environment: room with dimensions 4 m x 4 m x 3 m
% Scene: LED1(1m, 1m, 3m), LED2(3m, 1m, 3m), LED3(1m, 3m, 3m), LED4(3m, 3m, 3m)

%Each row of data below includes 50 values, corresponding to multiple positioning errors measured at the same position.
%Each column of data below includes 40000 positioning errors, corresponding to the simulation results of different positions in the simulation environment with the size of 200cm * 200cm. 
%Specifically, the value of the i-th row of each column of data represents the positioning error of the coordinate (x=ceil(i/200), y=i-200*(ceil(i/200)-1)).

VLP_error_LS_o=readmatrix('position_error_LS.csv'); % Average positioning error of LS
VLP_error_NLS_o=readmatrix('position_error_NLS.csv'); % Average positioning error of NLS
VLP_error_AWKNN_o=readmatrix('position_error_AWKNN.csv'); % Average positioning error of AWKNN
VLP_error_WKNN_o=readmatrix('position_error_WKNN.csv'); % Average positioning error of WKNN
%% 2D Distribution of the average positioning error of different methods

VLP_error_LS=reshape(mean(VLP_error_LS_o,2),200,200);
VLP_error_NLS=reshape(mean(VLP_error_NLS_o,2),200,200);
VLP_error_AWKNN=reshape(mean(VLP_error_AWKNN_o,2),200,200);
VLP_error_WKNN=reshape(mean(VLP_error_WKNN_o,2),200,200);

[xla,yla]=meshgrid(1:200,1:200);

figure(11)
s = surfc(xla,yla,VLP_error_LS);
s(1).EdgeColor = 'none';
grid on
xlabel('X/cm')
ylabel('Y/cm')
zlabel('Average error/cm')
view([50,34])
title('LS')
colorbar

figure(12)
s = surfc(xla,yla,VLP_error_NLS);
grid on
s(1).EdgeColor = 'none';
colorbar
xlabel('X/cm')
ylabel('Y/cm')
zlabel('Average error/cm')
view([50,34])
title('NLS')

figure(13)
s = surfc(xla,yla,VLP_error_WKNN);
s(1).EdgeColor = 'none';
grid on
colorbar
xlabel('X/cm')
ylabel('Y/cm')
zlabel('Average error/cm')
view([50,34])
title('WKNN')

figure(14)
s = surfc(xla,yla,VLP_error_AWKNN);
s(1).EdgeColor = 'none';
grid on
colorbar
xlabel('X/cm')
ylabel('Y/cm')
zlabel('Average error/cm')
view([50,34])
title('AWKNN')



%%  CDF curves of the positioning errors of different methods

%--------------------------------------------------------------
% Entire area
error_LS_Entire=reshape(VLP_error_LS_o,1,[]);
error_NLS_Entire=reshape(VLP_error_NLS_o,1,[]);
error_AWKNN_Entire=reshape(VLP_error_AWKNN_o,1,[]);
error_WKNN_Entire=reshape(VLP_error_WKNN_o,1,[]);


posi_num=length(error_LS_Entire);
UPBOUND_Entire=floor(max(error_LS_Entire));
CDF_LS_Entire=zeros(1,UPBOUND_Entire);
CDF_NLS_Entire=zeros(1,UPBOUND_Entire);
CDF_AWKNN_Entire=zeros(1,UPBOUND_Entire);
CDF_WKNN_Entire=zeros(1,UPBOUND_Entire);

for jj=1:UPBOUND_Entire
    CDF_LS_Entire(1,jj)=length(find(error_LS_Entire<=jj))/(posi_num);
    CDF_NLS_Entire(1,jj)=length(find(error_NLS_Entire<=jj))/(posi_num);    
    CDF_AWKNN_Entire(1,jj)=length(find(error_AWKNN_Entire<=jj))/(posi_num);
    CDF_WKNN_Entire(1,jj)=length(find(error_WKNN_Entire<=jj))/(posi_num);
end


%-----------------------------------------------------------------
% Central area 
 
INDX_Central=[];
for i=51:200
    for j=51:200
        INDX_Central=[INDX_Central (i-1)*200+j];
    end
end

error_LS_Inner=reshape(VLP_error_LS_o(INDX_Central,:),1,[]);
error_NLS_Inner=reshape(VLP_error_NLS_o(INDX_Central,:),1,[]);
error_AWKNN_Inner=reshape(VLP_error_AWKNN_o(INDX_Central,:),1,[]);
error_WKNN_Inner=reshape(VLP_error_WKNN_o(INDX_Central,:),1,[]);

posi_num=length(error_LS_Inner);
UPBOUND_Inner=floor(max(error_LS_Inner));
CDF_LS_Inner=zeros(1,UPBOUND_Inner);
CDF_NLS_Inner=zeros(1,UPBOUND_Inner);
CDF_AWKNN_Inner=zeros(1,UPBOUND_Inner);
CDF_WKNN_Inner=zeros(1,UPBOUND_Inner);

for jj=1:UPBOUND_Inner
    CDF_LS_Inner(1,jj)=length(find(error_LS_Inner<=jj))/(posi_num);
    CDF_NLS_Inner(1,jj)=length(find(error_NLS_Inner<=jj))/(posi_num);    
    CDF_AWKNN_Inner(1,jj)=length(find(error_AWKNN_Inner<=jj))/(posi_num);
    CDF_WKNN_Inner(1,jj)=length(find(error_WKNN_Inner<=jj))/(posi_num);
end

%-----------------------------------------------------------------
% Edge area 
INDX_Edge=1:200*200;
INDX_Edge(INDX_Central)=[];


error_LS_Edge=reshape(VLP_error_LS_o(INDX_Edge,:),1,[]);
error_NLS_Edge=reshape(VLP_error_NLS_o(INDX_Edge,:),1,[]);
error_AWKNN_Edge=reshape(VLP_error_AWKNN_o(INDX_Edge,:),1,[]);
error_WKNN_Edge=reshape(VLP_error_WKNN_o(INDX_Edge,:),1,[]);

posi_num=length(error_LS_Edge);
UPBOUND_Edge=floor(max(error_LS_Edge));
CDF_LS_Edge=zeros(1,UPBOUND_Edge);
CDF_NLS_Edge=zeros(1,UPBOUND_Edge);
CDF_AWKNN_Edge=zeros(1,UPBOUND_Edge);
CDF_WKNN_Edge=zeros(1,UPBOUND_Edge);
 
for jj=1:UPBOUND_Edge
    CDF_LS_Edge(1,jj)=length(find(error_LS_Edge<=jj))/(posi_num);
    CDF_NLS_Edge(1,jj)=length(find(error_NLS_Edge<=jj))/(posi_num);    
    CDF_AWKNN_Edge(1,jj)=length(find(error_AWKNN_Edge<=jj))/(posi_num);
    CDF_WKNN_Edge(1,jj)=length(find(error_WKNN_Edge<=jj))/(posi_num);    
     
end

 


figure(21)
plot(1:UPBOUND_Entire,CDF_LS_Entire,'-x',1:UPBOUND_Entire,CDF_NLS_Entire,'-o',1:UPBOUND_Entire,CDF_WKNN_Entire,'-s',1:UPBOUND_Entire,CDF_AWKNN_Entire,'-v','Linewidth',1.5);
legend('LS','NLS','WKNN','AWKNN')
xlabel('error/cm')
ylabel('CDF')
axis([1,12,0,1])
grid on
title('Entire room')

figure(22)
plot(1:UPBOUND_Inner,CDF_LS_Inner,'-x',1:UPBOUND_Inner,CDF_NLS_Inner,'-o',1:UPBOUND_Inner,CDF_WKNN_Inner,'-s',1:UPBOUND_Inner,CDF_AWKNN_Inner,'-v','Linewidth',1.5);
legend('LS','NLS','WKNN','AWKNN')
xlabel('error/cm')
ylabel('CDF')
axis([1,7,0,1])
grid on
title('Central area')

figure(23)
plot(1:UPBOUND_Edge,CDF_LS_Edge,'-x',1:UPBOUND_Edge,CDF_NLS_Edge,'-o',1:UPBOUND_Edge,CDF_WKNN_Edge,'-s',1:UPBOUND_Edge,CDF_AWKNN_Edge,'-v','Linewidth',1.5);
legend('LS','NLS','WKNN','AWKNN')
xlabel('error/cm')
ylabel('CDF')
axis([1,12,0,1])
grid on
title('Edge area')

%%  Histograms of the positioning errors of different methods

UPBOUNDx=floor(max(error_LS_Entire));
INDXX=0:0.2:UPBOUNDx;
for jj=2:length(INDXX)
    Hist_LS(1,jj)=length(find(error_LS_Entire<=INDXX(jj) & error_LS_Entire>INDXX(jj-1)));
    Hist_NLS(1,jj)=length(find(error_NLS_Entire<=INDXX(jj) & error_NLS_Entire>INDXX(jj-1)));    
    Hist_AWKNN(1,jj)=length(find(error_AWKNN_Entire<=INDXX(jj) & error_AWKNN_Entire>INDXX(jj-1)));
    Hist_WKNN(1,jj)=length(find(error_WKNN_Entire<=INDXX(jj) & error_WKNN_Entire>INDXX(jj-1)));
end

percent_LS=Hist_LS(2:end)/length(error_LS_Entire)*100;
percent_NLS=Hist_NLS(2:end)/length(error_LS_Entire)*100;
percent_AWKNN=Hist_AWKNN(2:end)/length(error_LS_Entire)*100;
percent_WKNN=Hist_WKNN(2:end)/length(error_LS_Entire)*100;



figure(3)
subplot(4,1,1)
colororder({'[0.00,0.45,0.74]','[0.47,0.67,0.19]'})
yyaxis left
bar(INDXX(2:end),Hist_LS(2:end),'FaceColor',[0.07,0.62,1.00],'EdgeColor',[0.47,0.67,0.19],'Linewidth',1)
xlabel('error/cm')
ylabel('Number of positioning error')
axis([0,11,0,2*10^5])
yyaxis right
bar(INDXX(2:end),percent_LS,'FaceColor',[0.07,0.62,1.00],'EdgeColor',[0.47,0.67,0.19],'Linewidth',1)
ylabel('Percentage/ % ')
title('LS')
grid on
yticks([0:2.5:10 ]) 
axis([0,11,0,10])


subplot(4,1,2)
colororder({'[0.00,0.45,0.74]','[0.47,0.67,0.19]'})
yyaxis left
bar(INDXX(2:end),Hist_NLS(2:end),'FaceColor',[0.07,0.62,1.00],'EdgeColor',[0.47,0.67,0.19],'Linewidth',1)
xlabel('error/cm')
ylabel('Number of positioning error')
title('NLS')
grid on
axis([0,11,0,2*10^5])
yyaxis right
bar(INDXX(2:end),percent_NLS,'FaceColor',[0.07,0.62,1.00],'EdgeColor',[0.47,0.67,0.19],'Linewidth',1)
ylabel('Percentage/ % ')
yticks([0:2.5:10 ]) 
axis([0,11,0,10])

subplot(4,1,3)
colororder({'[0.00,0.45,0.74]','[0.47,0.67,0.19]'})
yyaxis left
bar(INDXX(2:end),Hist_WKNN(2:end),'FaceColor',[0.07,0.62,1.00],'EdgeColor',[0.47,0.67,0.19],'Linewidth',1)
xlabel('error/cm')
ylabel('Number of positioning error')
title('WKNN')
axis([0,11,0,2*10^5])
grid on
yyaxis right
bar(INDXX(2:end),percent_WKNN,'FaceColor',[0.07,0.62,1.00],'EdgeColor',[0.47,0.67,0.19],'Linewidth',1)
ylabel('Percentage/ % ')
yticks([0:2.5:10 ]) 
axis([0,11,0,10])

subplot(4,1,4)
colororder({'[0.00,0.45,0.74]','[0.47,0.67,0.19]'})
yyaxis left
bar(INDXX(2:end),Hist_AWKNN(2:end),'FaceColor',[0.07,0.62,1.00],'EdgeColor',[0.47,0.67,0.19],'Linewidth',1)
xlabel('error/cm')
ylabel('Number of positioning error')
title('AWKNN')
axis([0,11,0,2*10^5])
grid on
yyaxis right
bar(INDXX(2:end),percent_AWKNN,'FaceColor',[0.07,0.62,1.00],'EdgeColor',[0.47,0.67,0.19],'Linewidth',1)
ylabel('Percentage/ % ')
yticks([0:2.5:10 ]) 
axis([0,11,0,10])


%%  positioning performance of different methods

%--------------------------------------------------------------
% Entire room
ME_Entire=mean([error_LS_Entire;error_WKNN_Entire;error_NLS_Entire;error_AWKNN_Entire;],2); % Positioning mean error
RMSE_Entire=sqrt(mean([error_LS_Entire.^2;error_WKNN_Entire.^2;error_NLS_Entire.^2;error_AWKNN_Entire.^2;],2)); % root mean square error (RMSE)

printmat([ME_Entire,RMSE_Entire], ' Entire area', 'LS WKNN NLS AWKNN', 'Mean/cm RMS/cm' )

%-----------------------------------------------------------------
% Inner area 
ME_Inner=mean([error_LS_Inner;error_WKNN_Inner;error_NLS_Inner;error_AWKNN_Inner;],2); % Positioning mean error
RMSE_Inner=sqrt(mean([error_LS_Inner.^2;error_WKNN_Inner.^2;error_NLS_Inner.^2;error_AWKNN_Inner.^2;],2)); % root mean square error (RMSE)

printmat([ME_Inner,RMSE_Inner], ' Central area', 'LS WKNN NLS AWKNN', 'Mean/cm RMS/cm' )
%-----------------------------------------------------------------
% Edge area 
ME_Edge=mean([error_LS_Edge;error_WKNN_Edge;error_NLS_Edge;error_AWKNN_Edge;],2); % Positioning mean error
RMSE_Edge=sqrt(mean([error_LS_Edge.^2;error_WKNN_Edge.^2;error_NLS_Edge.^2;error_AWKNN_Edge.^2;],2)); % root mean square error (RMSE)

printmat([ME_Edge,RMSE_Edge], ' Edge area', 'LS WKNN NLS AWKNN', 'Mean/cm RMS/cm' )

%%  Influence of K value on positioning accuracy

WKNN_diff_K_value=readmatrix('WKNN_diff_K_value.csv');
ME_diff_K=[mean(WKNN_diff_K_value)'; mean(error_AWKNN_Entire)];

figure(8)
b=bar(1:length(ME_diff_K)-1,roundn(ME_diff_K(1:end-1)',-2));
xtips1 = b(1).XEndPoints;
ytips1 = b(1).YEndPoints;
labels1 = string(b(1).YData);
text(xtips1,ytips1,labels1,'HorizontalAlignment','center',...
    'VerticalAlignment','bottom');
hold on
plot(0:length(ME_diff_K),ones(1,length(ME_diff_K)+1)*roundn(ME_diff_K(end),-2),'LineWidth',1.5)
xlabel('K value')
ylabel('Average positioning error/cm')
axis([0,length(ME_diff_K) 1.75,2.25])

toc




